@@ -112,7 +112,7 @@ setReplaceMethod("[", c("SpatRasterDataset","numeric","missing"),
 		stopifnot(inherits(value, "SpatRaster"))
 		i <- sort(i)
 		for (j in i) {
-			if (j == (size(x)+1)) {
+			if (j == (length(x)+1)) {
 				x@ptr$add(value@ptr, "", "", "", FALSE)
 			} else {
 				x@ptr$replace(j-1, value@ptr)
