@@ -39,6 +39,9 @@ setMethod("crs", signature("SpatRaster"),
 
 
 .txtCRS <- function(x, warn=TRUE) {
+	if (inherits(x, "SpatVector") | inherits(x, "SpatRaster")) {
+		x <- crs(x)
+	}
 	if (is.na(x)) {
 		x <- ""
 	} else if (inherits(x, "CRS")) {
@@ -104,7 +107,7 @@ setMethod("crs<-", signature("SpatVector", "ANY"),
 setMethod("is.lonlat", signature("SpatRaster"), 
 	function(x, perhaps=FALSE, warn=TRUE, global=FALSE) {
 		if (perhaps) {
-			ok <- x@ptr$isGeographic()
+			ok <- x@ptr$isLonLat()
 			if (ok) {
 				if (global) {
 					return(x@ptr$isGlobalLonLat())
@@ -123,7 +126,7 @@ setMethod("is.lonlat", signature("SpatRaster"),
 			}
 			return(ok)
 		} else {
-			ok <- x@ptr$isGeographic()
+			ok <- x@ptr$isLonLat()
 			if (ok && global) {
 				ok <- x@ptr$isGlobalLonLat()
 			}
@@ -135,15 +138,15 @@ setMethod("is.lonlat", signature("SpatRaster"),
 
 setMethod("is.lonlat", signature("SpatVector"), 
 	function(x, perhaps=FALSE, warn=TRUE) {
-		ok <- x@ptr$isGeographic()
+		ok <- x@ptr$isLonLat()
 		if (ok) return(ok)
 		if (perhaps) {
 			ok <- x@ptr$couldBeLonLat()
 			if (ok && warn) {
 				if (crs(x) == "") warn("is.lonlat", "assuming lon/lat crs")
 			}
-			ok
 		}
+		ok
 	}
 )
 
