@@ -66,12 +66,6 @@ setMethod("names", signature(x="SpatVector"),
 	}
 )
 
-setMethod("colnames", signature(x="SpatVector"), 
-	function(x) { 
-		x@ptr$names
-	}
-)
-
 setMethod("names<-", signature(x="SpatVector"), 
 	function(x, value)  {
 		if (length(value) != ncol(x)) {
@@ -85,12 +79,6 @@ setMethod("names<-", signature(x="SpatVector"),
 	}
 )
 
-setMethod("colnames<-", signature(x="SpatVector"), 
-	function(x, value)  {
-		`names<-`(x, value)
-	}
-)
-
 
 setMethod("varnames", signature(x="SpatRaster"), 
 	function(x) { 
