@@ -349,6 +349,15 @@
 setMethod("plot", signature(x="SpatRaster", y="numeric"), 
 	function(x, y=1, col, type, mar=NULL, legend=TRUE, axes=TRUE, plg=list(), pax=list(), maxcell=500000, smooth=FALSE, range=NULL, levels=NULL, fun=NULL, colNA=NULL, alpha=NULL, ...) {
 
+		y <- round(y)
+		stopifnot((min(y) > 0) & (max(y) <= nlyr(x)))
+
+		if (length(y) > 1) {
+			x <- x[[y]]
+			plot(x, col=col, type=type, mar=mar, legend=legend, axes=axes, plg=plg, pax=pax, maxcell=maxcell/(length(x)/2), smooth=smooth, range=range, levels=levels, fun=fun, colNA=colNA, alpha=alpha, ...)
+			return(invisible())
+		}
+
 		x <- x[[y]]
 		if (!hasValues(x)) { error("plot", "SpatRaster has no cell values") }
 		if (ncell(x) > 1.1 * maxcell) {
@@ -434,6 +443,7 @@ setMethod("plot", signature(x="SpatRaster", y="missing"),
 			main <- rep_len(main, nl)
 		}
 		x <- spatSample(x, maxcell, method="regular", as.raster=TRUE)
+		#if (onelegend) { legend <- FALSE }
 		for (i in 1:nl) {
 			plot(x, i, main=main[i], mar=mar, ...)
 		}
