@@ -80,6 +80,7 @@ class SpatRasterSource {
 		bool hasWindow=false;
 		SpatWindow window;
 	
+
 		bool multidim = false;
 		size_t m_ndims;
 		std::vector<size_t> m_dims;
@@ -141,11 +142,15 @@ class SpatRasterSource {
 
 //		std::vector<SpatRasterSource> subset(std::vector<unsigned> lyrs);
 		SpatRasterSource subset(std::vector<unsigned> lyrs);
-		std::vector<double> getValues(unsigned lyr);
+		void getValues(std::vector<double> &v, unsigned lyr);
+		void appendValues(std::vector<double> &v, unsigned lyr);
+		
 		void setRange();
 		void resize(unsigned n);
 		bool in_order();
 		bool combine_sources(const SpatRasterSource &x);
+		bool combine(SpatRasterSource &x);
+		
 
 		bool parameters_changed = false;		
 		
@@ -172,14 +177,9 @@ class SpatRaster {
 		bool gdal_minmax = true;
 
 	protected:
-		//SpatExtent extent;
 		SpatExtent window;
-		//SpatSRS srs;
 
 	public:
-	//	std::string prj;
-
-	//	bool GDALregistred = false;
 
 #ifdef useRcpp
 		Progress* pbar;
@@ -189,8 +189,6 @@ class SpatRaster {
 ////////////////////////////////////////////////////
 // properties and property-like methods for entire object
 ////////////////////////////////////////////////////
-
-		//std::string name;
 		
 		std::vector<SpatRasterSource> source;
 
@@ -238,7 +236,6 @@ class SpatRaster {
 #endif 
 */
 
-		bool is_geographic();
 		bool is_lonlat();
 		bool could_be_lonlat();
 		bool is_global_lonlat();
@@ -333,6 +330,8 @@ class SpatRaster {
 
 		void addSource(SpatRaster x);	
 		SpatRaster combineSources(SpatRaster x);
+		void combine(SpatRaster x);
+		
 		SpatRaster subset(std::vector<unsigned> lyrs, SpatOptions &opt);
 		SpatRaster replace(SpatRaster x, unsigned layer, SpatOptions &opt);
 ////////////////////////////////////////////////////
@@ -491,13 +490,15 @@ class SpatRaster {
 ////////////////////////////////////////////////////
 
 		SpatRaster collapse_sources();
+		void collapse();
+		
 		SpatRaster rectify(std::string method, SpatRaster aoi, unsigned useaoi, bool snap, SpatOptions &opt);
 		
         std::vector<std::vector<double>> adjacent(std::vector<double> cells, std::string directions, bool include);
  		SpatRaster aggregate(std::vector<unsigned> fact, std::string fun, bool narm, SpatOptions &opt);
 		SpatExtent align(SpatExtent e, std::string snap);
-		SpatRaster rst_area(bool adjust, bool mask, SpatOptions &opt);
-		std::vector<double> sum_area(bool adjust, SpatOptions &opt);
+		SpatRaster rst_area(bool mask, std::string unit, bool transform, SpatOptions &opt);
+		std::vector<double> sum_area(std::string unit, bool transform, SpatOptions &opt);
 		std::vector<std::vector<double>> area_by_value(SpatOptions &opt);
 
 		SpatRaster arith(SpatRaster x, std::string oper, SpatOptions &opt);
@@ -509,6 +510,7 @@ class SpatRaster {
 
 		SpatVector as_polygons(bool trunc, bool dissolve, bool values, bool narm, SpatOptions &opt);
 		SpatVector polygonize(bool trunc, bool values, bool narm, bool aggregate, SpatOptions &opt);
+		SpatVector as_lines();
 		SpatVector as_points(bool values, bool narm, SpatOptions &opt);
 		SpatRaster atan_2(SpatRaster x, SpatOptions &opt);
 
@@ -603,7 +605,7 @@ class SpatRaster {
 
 		SpatRaster rasterizeLyr(SpatVector x, double value, double background, bool touches, bool update, SpatOptions &opt);
 
-		SpatRaster rasterize(SpatVector x, std::string field, std::vector<double> values, double background, bool touches, bool add, bool weights, bool update, SpatOptions &opt);
+		SpatRaster rasterize(SpatVector x, std::string field, std::vector<double> values, double background, bool touches, bool add, bool weights, bool update, bool minmax, SpatOptions &opt);
 		std::vector<double> rasterizeCells(SpatVector &v, bool touches);
 		std::vector<std::vector<double>> rasterizeCellsWeights(SpatVector &v, bool touches);
 
